@@ -17,12 +17,12 @@
 # along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
 
 import gobject
-import glib
 import gnomedvb
-import gtk
+from gi.repository import Gtk
 from gettext import gettext as _
 from gnomedvb.ui.wizard import DVB_TYPE_TO_DESC
 from gnomedvb.ui.wizard.pages.BasePage import BasePage
+from gnomedvb.ui.widgets.Frame import BaseFrame
 
 class AdaptersPage(BasePage):
     
@@ -39,18 +39,14 @@ class AdaptersPage(BasePage):
         self.__model = model
         self._progressbar = None
         self.devicesview = None
-        self.scrolledview = None
-        
-        self._label = gtk.Label()
-        self._label.set_line_wrap(True)
-        self.pack_start(self._label)
+        self.frame = None
         
         # Name, Type Name, Type, adapter, frontend, registered
-        self.deviceslist = gtk.ListStore(str, str, str, int, int, bool)
+        self.deviceslist = Gtk.ListStore(str, str, str, int, int, bool)
         
     def show_no_devices(self):
-        if self.scrolledview:
-            self.scrolledview.hide()
+        if self.frame:
+            self.frame.hide()
     
         text = "<big><span weight=\"bold\">%s</span></big>" % _('No devices have been found.')
         text += "\n\n"
@@ -59,41 +55,44 @@ class AdaptersPage(BasePage):
         self._label.show()
     
     def show_devices(self):
-        text = _("Select device you want to configure.")
-        self._label.set_markup (text)
-        self._label.show()
-    
+        self._label.hide()
+
         if self.devicesview == None:
-            self.devicesview = gtk.TreeView(self.deviceslist)
+            self.devicesview = Gtk.TreeView.new_with_model(self.deviceslist)
             self.devicesview.get_selection().connect("changed",
                 self.on_device_selection_changed)
         
-            cell_name = gtk.CellRendererText()
-            col_name = gtk.TreeViewColumn(_("Name"))
-            col_name.pack_start(cell_name)
-            col_name.add_attribute(cell_name, "text", 0)
+            cell_name = Gtk.CellRendererText()
+            col_name = Gtk.TreeViewColumn(_("Name"))
+            col_name.pack_start(cell_name, True)
+            col_name.set_cell_data_func(cell_name, self.name_data_func, None)
             self.devicesview.append_column(col_name)
         
-            cell_type = gtk.CellRendererText()
-            col_type = gtk.TreeViewColumn(_("Type"))
-            col_type.pack_start(cell_type)
+            cell_type = Gtk.CellRendererText()
+            col_type = Gtk.TreeViewColumn(_("Type"))
+            col_type.pack_start(cell_type, True)
             col_type.add_attribute(cell_type, "text", 1)
             self.devicesview.append_column(col_type)
         
-            self.scrolledview = gtk.ScrolledWindow()
-            self.scrolledview.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-            self.scrolledview.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-            self.scrolledview.add(self.devicesview)
-            self.scrolledview.show_all()
+            scrolledview = Gtk.ScrolledWindow()
+            scrolledview.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+            scrolledview.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+            scrolledview.add(self.devicesview)
+            scrolledview.show_all()
         
-            self.pack_start(self.scrolledview)
+            text = "<b>%s</b>" % _("Select the device you want to configure.")
+            self.frame = BaseFrame(text, scrolledview)
+            self.frame.show()
+            self.pack_start(self.frame, True, True, 0)
+
+        self.devicesview.grab_focus()
         
         if len(self.deviceslist) == 1:
             self.emit("next-page")
         
     def show_all_configured(self):
-        if self.scrolledview:
-            self.scrolledview.hide()
+        if self.frame:
+            self.frame.hide()
 
         text = "<big><span weight=\"bold\">%s</span></big>" % _('All devices are already configured.')
         text += "\n\n"
@@ -101,21 +100,35 @@ class AdaptersPage(BasePage):
         self._label.set_markup (text)
         self._label.show()
         
+    def show_error(self, error):
+        if self.frame:
+            self.frame.hide()
+            
+        text = "<big><span weight=\"bold\">%s</span></big>" % _('An error occured while retrieving devices.')
+        text += "\n\n"
+        text += _("Make sure other applications don't access DVB devices and you have permissions to access them.")
+        text += "\n\n"
+        text += _('The detailed error message is:')
+        text += "\n<i>%s</i>" % error
+        self._label.set_selectable(True)
+        self._label.set_markup (text)
+        self._label.show()
+        
     def show_progressbar(self):
         self._label.hide()
-        self._progressbar = gtk.ProgressBar()
+        self._progressbar = Gtk.ProgressBar()
         self._progressbar.set_text(_("Searching for devices"))
         self._progressbar.set_fraction(0.1)
         self._progressbar.show()
-        self.pack_start(self._progressbar, False)
-        self._progressbar_timer = glib.timeout_add(100, self.progressbar_pulse)
+        self.pack_start(self._progressbar, False, True, 0)
+        self._progressbar_timer = gobject.timeout_add(100, self.progressbar_pulse, None)
         
     def destroy_progressbar(self):
-        glib.source_remove(self._progressbar_timer)
+        gobject.source_remove(self._progressbar_timer)
         self._progressbar_timer = None
         self._progressbar.destroy()
 
-    def progressbar_pulse(self):
+    def progressbar_pulse(self, user_data):
         self._progressbar.pulse()
         return True
 
@@ -153,20 +166,30 @@ class AdaptersPage(BasePage):
                     dev.type_name = DVB_TYPE_TO_DESC[dev.type]
                     dev.registered = True
                     registered.add(dev)       
-            self.__model.get_all_devices(reply_handler=devices_handler)
+            self.__model.get_all_devices(result_handler=devices_handler)
         
         def devices_handler(devices):
+            error = None
             for dev in devices:
                 if dev not in registered:
-                    info = gnomedvb.get_adapter_info(dev.adapter)
-                    dev.name = info["name"]
-                    dev.type = info["type"]
-                    dev.type_name = DVB_TYPE_TO_DESC[info["type"]]
-                    dev.registered = False
-                    unregistered.add(dev)
-               
+                    success, info = gnomedvb.get_adapter_info(dev.adapter,
+                        dev.frontend)
+                    if success:
+                        dev.name = info["name"]
+                        dev.type = info["type"]
+                        if info["type"] in DVB_TYPE_TO_DESC:
+                            dev.type_name = DVB_TYPE_TO_DESC[info["type"]]
+                        else:
+                            dev.type_name = info["type"]
+                        dev.registered = False
+                        unregistered.add(dev)
+                    else:
+                        error = info
+
+            all_devs = registered | unregistered
+            has_device = len(all_devs) > 0
             if self.__use_configured:
-                devs =  registered | unregistered
+                devs = all_devs
             else:
                 devs = unregistered
 
@@ -176,9 +199,11 @@ class AdaptersPage(BasePage):
                     dev.type, dev.adapter, dev.frontend, dev.registered])
 
             self.destroy_progressbar()
-
-            if len(devs) == 0:
-                if self.__use_configured:
+            
+            if error != None:
+                self.show_error(error)
+            elif len(devs) == 0:
+                if not has_device:
                     self.show_no_devices()
                 else:
                     self.show_all_configured()
@@ -190,7 +215,7 @@ class AdaptersPage(BasePage):
         self.__adapter_info = None
         self.show_progressbar()
         
-        self.__model.get_registered_device_groups(reply_handler=registered_handler)
+        self.__model.get_registered_device_groups(result_handler=registered_handler)
     
     def on_device_selection_changed(self, treeselection):
         model, aiter = treeselection.get_selected()
@@ -203,4 +228,14 @@ class AdaptersPage(BasePage):
             self.emit("finished", True)
         else:
             self.emit("finished", False)
+
+    def name_data_func(self, column, cell, model, aiter, user_data):
+        name = model[aiter][0]
+        adapter = model[aiter][3]
+        frontend = model[aiter][4]
+
+        text = _("<b>%s</b>\n") % name
+        text += "<small>%s</small>" % (_("Adapter: %d, Frontend: %d") % (adapter, frontend))
+
+        cell.set_property("markup", text)
         
